package de.urszeidler.eclipse.callchain.diagram.edit.policies;

import java.util.Collections;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.gef.commands.UnexecutableCommand;
import org.eclipse.gef.requests.ReconnectRequest;
import org.eclipse.gmf.runtime.common.core.command.ICommand;
import org.eclipse.gmf.runtime.diagram.core.commands.DeleteCommand;
import org.eclipse.gmf.runtime.diagram.core.util.ViewUtil;
import org.eclipse.gmf.runtime.diagram.ui.commands.CommandProxy;
import org.eclipse.gmf.runtime.diagram.ui.commands.ICommandProxy;
import org.eclipse.gmf.runtime.diagram.ui.editparts.IGraphicalEditPart;
import org.eclipse.gmf.runtime.diagram.ui.editpolicies.SemanticEditPolicy;
import org.eclipse.gmf.runtime.diagram.ui.requests.EditCommandRequestWrapper;
import org.eclipse.gmf.runtime.emf.commands.core.command.CompositeTransactionalCommand;
import org.eclipse.gmf.runtime.emf.type.core.ElementTypeRegistry;
import org.eclipse.gmf.runtime.emf.type.core.IEditHelperContext;
import org.eclipse.gmf.runtime.emf.type.core.IElementType;
import org.eclipse.gmf.runtime.emf.type.core.requests.ConfigureRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.CreateElementRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.CreateRelationshipRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.DestroyElementRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.DestroyReferenceRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.DestroyRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.DuplicateElementsRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.GetEditContextRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.IEditCommandRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.MoveRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.ReorientReferenceRelationshipRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.ReorientRelationshipRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.SetRequest;
import org.eclipse.gmf.runtime.notation.Edge;
import org.eclipse.gmf.runtime.notation.View;

import de.urszeidler.eclipse.callchain.Artifact;
import de.urszeidler.eclipse.callchain.AtlLibary;
import de.urszeidler.eclipse.callchain.AtlRes;
import de.urszeidler.eclipse.callchain.Call;
import de.urszeidler.eclipse.callchain.Callable;
import de.urszeidler.eclipse.callchain.CallchainPackage;
import de.urszeidler.eclipse.callchain.Calls;
import de.urszeidler.eclipse.callchain.Comment;
import de.urszeidler.eclipse.callchain.Documentable;
import de.urszeidler.eclipse.callchain.Generator;
import de.urszeidler.eclipse.callchain.Model;
import de.urszeidler.eclipse.callchain.diagram.edit.helpers.CallchainBaseEditHelper;
import de.urszeidler.eclipse.callchain.diagram.expressions.CallchainAbstractExpression;
import de.urszeidler.eclipse.callchain.diagram.expressions.CallchainOCLFactory;
import de.urszeidler.eclipse.callchain.diagram.part.CallchainDiagramEditorPlugin;
import de.urszeidler.eclipse.callchain.diagram.part.CallchainVisualIDRegistry;

/**
 * @generated
 */
public class CallchainBaseItemSemanticEditPolicy extends SemanticEditPolicy {

	/**
	 * Extended request data key to hold editpart visual id.
	 * 
	 * @generated
	 */
	public static final String VISUAL_ID_KEY = "visual_id"; //$NON-NLS-1$

	/**
	 * Extended request data key to hold editpart visual id.
	 * Add visual id of edited editpart to extended data of the request
	 * so command switch can decide what kind of diagram element is being edited.
	 * It is done in those cases when it's not possible to deduce diagram
	 * element kind from domain element.
	 * 
	 * @generated
	 */
	public Command getCommand(Request request) {
		if (request instanceof ReconnectRequest) {
			Object view = ((ReconnectRequest) request).getConnectionEditPart().getModel();
			if (view instanceof View) {
				Integer id = new Integer(CallchainVisualIDRegistry.getVisualID((View) view));
				request.getExtendedData().put(VISUAL_ID_KEY, id);
			}
		}
		return super.getCommand(request);
	}

	/**
	 * Returns visual id from request parameters.
	 * 
	 * @generated
	 */
	protected int getVisualID(IEditCommandRequest request) {
		Object id = request.getParameter(VISUAL_ID_KEY);
		return id instanceof Integer ? ((Integer) id).intValue() : -1;
	}

	/**
	 * @generated
	 */
	protected Command getSemanticCommand(IEditCommandRequest request) {
		IEditCommandRequest completedRequest = completeRequest(request);
		Object editHelperContext = completedRequest.getEditHelperContext();
		if (editHelperContext instanceof View
				|| (editHelperContext instanceof IEditHelperContext && ((IEditHelperContext) editHelperContext)
						.getEObject() instanceof View)) {
			// no semantic commands are provided for pure design elements
			return null;
		}
		if (editHelperContext == null) {
			editHelperContext = ViewUtil.resolveSemanticElement((View) getHost().getModel());
		}
		IElementType elementType = ElementTypeRegistry.getInstance().getElementType(editHelperContext);
		if (elementType == ElementTypeRegistry.getInstance().getType("org.eclipse.gmf.runtime.emf.type.core.default")) { //$NON-NLS-1$ 
			elementType = null;
		}
		Command semanticCommand = getSemanticCommandSwitch(completedRequest);
		if (elementType != null) {
			if (semanticCommand != null) {
				ICommand command = semanticCommand instanceof ICommandProxy ? ((ICommandProxy) semanticCommand)
						.getICommand() : new CommandProxy(semanticCommand);
				completedRequest.setParameter(CallchainBaseEditHelper.EDIT_POLICY_COMMAND, command);
			}
			ICommand command = elementType.getEditCommand(completedRequest);
			if (command != null) {
				if (!(command instanceof CompositeTransactionalCommand)) {
					TransactionalEditingDomain editingDomain = ((IGraphicalEditPart) getHost()).getEditingDomain();
					command = new CompositeTransactionalCommand(editingDomain, command.getLabel()).compose(command);
				}
				semanticCommand = new ICommandProxy(command);
			}
		}
		boolean shouldProceed = true;
		if (completedRequest instanceof DestroyRequest) {
			shouldProceed = shouldProceed((DestroyRequest) completedRequest);
		}
		if (shouldProceed) {
			if (completedRequest instanceof DestroyRequest) {
				TransactionalEditingDomain editingDomain = ((IGraphicalEditPart) getHost()).getEditingDomain();
				Command deleteViewCommand = new ICommandProxy(new DeleteCommand(editingDomain, (View) getHost()
						.getModel()));
				semanticCommand = semanticCommand == null ? deleteViewCommand : semanticCommand
						.chain(deleteViewCommand);
			}
			return semanticCommand;
		}
		return null;
	}

	/**
	 * @generated
	 */
	protected Command getSemanticCommandSwitch(IEditCommandRequest req) {
		if (req instanceof CreateRelationshipRequest) {
			return getCreateRelationshipCommand((CreateRelationshipRequest) req);
		} else if (req instanceof CreateElementRequest) {
			return getCreateCommand((CreateElementRequest) req);
		} else if (req instanceof ConfigureRequest) {
			return getConfigureCommand((ConfigureRequest) req);
		} else if (req instanceof DestroyElementRequest) {
			return getDestroyElementCommand((DestroyElementRequest) req);
		} else if (req instanceof DestroyReferenceRequest) {
			return getDestroyReferenceCommand((DestroyReferenceRequest) req);
		} else if (req instanceof DuplicateElementsRequest) {
			return getDuplicateCommand((DuplicateElementsRequest) req);
		} else if (req instanceof GetEditContextRequest) {
			return getEditContextCommand((GetEditContextRequest) req);
		} else if (req instanceof MoveRequest) {
			return getMoveCommand((MoveRequest) req);
		} else if (req instanceof ReorientReferenceRelationshipRequest) {
			return getReorientReferenceRelationshipCommand((ReorientReferenceRelationshipRequest) req);
		} else if (req instanceof ReorientRelationshipRequest) {
			return getReorientRelationshipCommand((ReorientRelationshipRequest) req);
		} else if (req instanceof SetRequest) {
			return getSetCommand((SetRequest) req);
		}
		return null;
	}

	/**
	 * @generated
	 */
	protected Command getConfigureCommand(ConfigureRequest req) {
		return null;
	}

	/**
	 * @generated
	 */
	protected Command getCreateRelationshipCommand(CreateRelationshipRequest req) {
		return null;
	}

	/**
	 * @generated
	 */
	protected Command getCreateCommand(CreateElementRequest req) {
		return null;
	}

	/**
	 * @generated
	 */
	protected Command getSetCommand(SetRequest req) {
		return null;
	}

	/**
	 * @generated
	 */
	protected Command getEditContextCommand(GetEditContextRequest req) {
		return null;
	}

	/**
	 * @generated
	 */
	protected Command getDestroyElementCommand(DestroyElementRequest req) {
		return null;
	}

	/**
	 * @generated
	 */
	protected Command getDestroyReferenceCommand(DestroyReferenceRequest req) {
		return null;
	}

	/**
	 * @generated
	 */
	protected Command getDuplicateCommand(DuplicateElementsRequest req) {
		return null;
	}

	/**
	 * @generated
	 */
	protected Command getMoveCommand(MoveRequest req) {
		return null;
	}

	/**
	 * @generated
	 */
	protected Command getReorientReferenceRelationshipCommand(ReorientReferenceRelationshipRequest req) {
		return UnexecutableCommand.INSTANCE;
	}

	/**
	 * @generated
	 */
	protected Command getReorientRelationshipCommand(ReorientRelationshipRequest req) {
		return UnexecutableCommand.INSTANCE;
	}

	/**
	 * @generated
	 */
	protected final Command getGEFWrapper(ICommand cmd) {
		return new ICommandProxy(cmd);
	}

	/**
	 * @deprecated use getGEFWrapper() instead
	 * @generated
	 */
	protected final Command getMSLWrapper(ICommand cmd) {
		// XXX deprecated: use getGEFWrapper() instead
		return getGEFWrapper(cmd);
	}

	/**
	 * @generated
	 */
	protected EObject getSemanticElement() {
		return ViewUtil.resolveSemanticElement((View) getHost().getModel());
	}

	/**
	 * Returns editing domain from the host edit part.
	 * 
	 * @generated
	 */
	protected TransactionalEditingDomain getEditingDomain() {
		return ((IGraphicalEditPart) getHost()).getEditingDomain();
	}

	/**
	 * Creates command to destroy the link.
	 * 
	 * @generated
	 */
	protected Command getDestroyElementCommand(View view) {
		EditPart editPart = (EditPart) getHost().getViewer().getEditPartRegistry().get(view);
		DestroyElementRequest request = new DestroyElementRequest(getEditingDomain(), false);
		return editPart.getCommand(new EditCommandRequestWrapper(request, Collections.EMPTY_MAP));
	}

	/**
	 * Creates commands to destroy all host incoming and outgoing links.
	 * 
	 * @generated
	 */
	protected CompoundCommand getDestroyEdgesCommand() {
		CompoundCommand cmd = new CompoundCommand();
		View view = (View) getHost().getModel();
		for (Iterator it = view.getSourceEdges().iterator(); it.hasNext();) {
			cmd.add(getDestroyElementCommand((Edge) it.next()));
		}
		for (Iterator it = view.getTargetEdges().iterator(); it.hasNext();) {
			cmd.add(getDestroyElementCommand((Edge) it.next()));
		}
		return cmd;
	}

	/**
	 * @generated
	 */
	protected void addDestroyShortcutsCommand(CompoundCommand command) {
		View view = (View) getHost().getModel();
		if (view.getEAnnotation("Shortcut") != null) { //$NON-NLS-1$
			return;
		}
		for (Iterator it = view.getDiagram().getChildren().iterator(); it.hasNext();) {
			View nextView = (View) it.next();
			if (nextView.getEAnnotation("Shortcut") == null || !nextView.isSetElement() || nextView.getElement() != view.getElement()) { //$NON-NLS-1$
				continue;
			}
			command.add(getDestroyElementCommand(nextView));
		}
	}

	/**
	 * @generated
	 */
	public static class LinkConstraints {
		/**
		 * @generated
		 */
		private static final String OPPOSITE_END_VAR = "oppositeEnd"; //$NON-NLS-1$

		/**
		 * @generated
		 */
		private static CallchainAbstractExpression ModelAlias_4016_SourceExpression;

		/**
		 * @generated
		 */
		private static CallchainAbstractExpression ModelAlias_4016_TargetExpression;

		/**
		 * @generated
		 */
		private static CallchainAbstractExpression MMAlias_4017_SourceExpression;

		/**
		 * @generated
		 */
		private static CallchainAbstractExpression MMAlias_4017_TargetExpression;

		/**
		 * @generated
		 */
		private static CallchainAbstractExpression AtlResSuperimpose_4019_SourceExpression;

		/**
		 * @generated
		 */
		private static CallchainAbstractExpression AtlResSuperimpose_4019_TargetExpression;

		/**
		 * @generated
		 */
		private static CallchainAbstractExpression CallIn_4020_SourceExpression;

		/**
		 * @generated
		 */
		private static CallchainAbstractExpression CallIn_4020_TargetExpression;

		/**
		 * @generated
		 */
		private static CallchainAbstractExpression GeneratorUses_4026_SourceExpression;

		/**
		 * @generated
		 */
		private static CallchainAbstractExpression GeneratorUses_4026_TargetExpression;

		/**
		 * @generated
		 */
		private static CallchainAbstractExpression GeneratorProduce_4027_SourceExpression;

		/**
		 * @generated
		 */
		private static CallchainAbstractExpression GeneratorProduce_4027_TargetExpression;

		/**
		 * @generated
		 */
		private static CallchainAbstractExpression CallableNext_4028_SourceExpression;

		/**
		 * @generated
		 */
		private static CallchainAbstractExpression CallableNext_4028_TargetExpression;

		/**
		 * @generated
		 */
		public static boolean canCreateModelAlias_4016(Call source, Model target) {
			return canExistModelAlias_4016(source, target);
		}

		/**
		 * @generated
		 */
		public static boolean canCreateMMAlias_4017(Calls source, Model target) {
			return canExistMMAlias_4017(source, target);
		}

		/**
		 * @generated
		 */
		public static boolean canCreateModelMm_4018(Model source, de.urszeidler.eclipse.callchain.MM target) {
			if (source != null) {
				if (source.getMm() != null) {
					return false;
				}
			}
			return canExistModelMm_4018(source, target);
		}

		/**
		 * @generated
		 */
		public static boolean canCreateAtlResSuperimpose_4019(AtlRes source, AtlRes target) {
			if (source != null) {
				if (source.getSuperimpose().contains(target)) {
					return false;
				}
			}
			return canExistAtlResSuperimpose_4019(source, target);
		}

		/**
		 * @generated
		 */
		public static boolean canCreateCallIn_4020(Call source, Model target) {
			if (source != null) {
				if (source.getIn().contains(target)) {
					return false;
				}
			}
			return canExistCallIn_4020(source, target);
		}

		/**
		 * @generated
		 */
		public static boolean canCreateCallOut_4021(Call source, Model target) {
			if (source != null) {
				if (source.getOut().contains(target)) {
					return false;
				}
			}
			return canExistCallOut_4021(source, target);
		}

		/**
		 * @generated
		 */
		public static boolean canCreateCallRes_4022(Call source, AtlRes target) {
			if (source != null) {
				if (source.getRes() != null) {
					return false;
				}
			}
			return canExistCallRes_4022(source, target);
		}

		/**
		 * @generated
		 */
		public static boolean canCreateCallLibarys_4023(Call source, AtlLibary target) {
			if (source != null) {
				if (source.getLibarys().contains(target)) {
					return false;
				}
			}
			return canExistCallLibarys_4023(source, target);
		}

		/**
		 * @generated
		 */
		public static boolean canCreateCallArtifact_4024(Call source, Artifact target) {
			if (source != null) {
				if (source.getArtifact() != null) {
					return false;
				}
			}
			return canExistCallArtifact_4024(source, target);
		}

		/**
		 * @generated
		 */
		public static boolean canCreateCommentDocumentedElement_4025(Comment source, Documentable target) {
			if (source != null) {
				if (source.getDocumentedElement().contains(target)) {
					return false;
				}
			}
			return canExistCommentDocumentedElement_4025(source, target);
		}

		/**
		 * @generated
		 */
		public static boolean canCreateGeneratorUses_4026(Generator source, Artifact target) {
			if (source != null) {
				if (source.getUses().contains(target)) {
					return false;
				}
			}
			return canExistGeneratorUses_4026(source, target);
		}

		/**
		 * @generated
		 */
		public static boolean canCreateGeneratorProduce_4027(Generator source, Artifact target) {
			if (source != null) {
				if (source.getProduce().contains(target)) {
					return false;
				}
			}
			return canExistGeneratorProduce_4027(source, target);
		}

		/**
		 * @generated
		 */
		public static boolean canCreateCallableNext_4028(Callable source, Callable target) {
			if (source != null) {
				if (source.getNext() != null) {
					return false;
				}
			}
			return canExistCallableNext_4028(source, target);
		}

		/**
		 * @generated
		 */
		public static boolean canExistModelAlias_4016(Call source, Model target) {
			try {
				if (source == null) {
					return true;
				}
				if (ModelAlias_4016_SourceExpression == null) {
					Map env = Collections.singletonMap(OPPOSITE_END_VAR, CallchainPackage.eINSTANCE.getModel());
					ModelAlias_4016_SourceExpression = CallchainOCLFactory.getExpression(
							"true", CallchainPackage.eINSTANCE.getCall(), env); //$NON-NLS-1$
				}
				Object sourceVal = ModelAlias_4016_SourceExpression.evaluate(source, Collections.singletonMap(
						OPPOSITE_END_VAR, target));
				if (false == sourceVal instanceof Boolean || !((Boolean) sourceVal).booleanValue()) {
					return false;
				} // else fall-through
				if (target == null) {
					return true;
				}
				if (ModelAlias_4016_TargetExpression == null) {
					Map env = Collections.singletonMap(OPPOSITE_END_VAR, CallchainPackage.eINSTANCE.getCall());
					ModelAlias_4016_TargetExpression = CallchainOCLFactory.getExpression(
							"not oppositeEnd.oclIsTypeOf(ModelAlias)", CallchainPackage.eINSTANCE.getModel(), env); //$NON-NLS-1$
				}
				Object targetVal = ModelAlias_4016_TargetExpression.evaluate(target, Collections.singletonMap(
						OPPOSITE_END_VAR, source));
				if (false == targetVal instanceof Boolean || !((Boolean) targetVal).booleanValue()) {
					return false;
				} // else fall-through
				return true;
			} catch (Exception e) {
				CallchainDiagramEditorPlugin.getInstance().logError("Link constraint evaluation error", e); //$NON-NLS-1$
				return false;
			}
		}

		/**
		 * @generated
		 */
		public static boolean canExistMMAlias_4017(Calls source, Model target) {
			try {
				if (source == null) {
					return true;
				}
				if (MMAlias_4017_SourceExpression == null) {
					Map env = Collections.singletonMap(OPPOSITE_END_VAR, CallchainPackage.eINSTANCE.getModel());
					MMAlias_4017_SourceExpression = CallchainOCLFactory.getExpression(
							"true", CallchainPackage.eINSTANCE.getCalls(), env); //$NON-NLS-1$
				}
				Object sourceVal = MMAlias_4017_SourceExpression.evaluate(source, Collections.singletonMap(
						OPPOSITE_END_VAR, target));
				if (false == sourceVal instanceof Boolean || !((Boolean) sourceVal).booleanValue()) {
					return false;
				} // else fall-through
				if (target == null) {
					return true;
				}
				if (MMAlias_4017_TargetExpression == null) {
					Map env = Collections.singletonMap(OPPOSITE_END_VAR, CallchainPackage.eINSTANCE.getCalls());
					MMAlias_4017_TargetExpression = CallchainOCLFactory.getExpression(
							"true", CallchainPackage.eINSTANCE.getModel(), env); //$NON-NLS-1$
				}
				Object targetVal = MMAlias_4017_TargetExpression.evaluate(target, Collections.singletonMap(
						OPPOSITE_END_VAR, source));
				if (false == targetVal instanceof Boolean || !((Boolean) targetVal).booleanValue()) {
					return false;
				} // else fall-through
				return true;
			} catch (Exception e) {
				CallchainDiagramEditorPlugin.getInstance().logError("Link constraint evaluation error", e); //$NON-NLS-1$
				return false;
			}
		}

		/**
		 * @generated
		 */
		public static boolean canExistModelMm_4018(Model source, de.urszeidler.eclipse.callchain.MM target) {

			return true;
		}

		/**
		 * @generated
		 */
		public static boolean canExistAtlResSuperimpose_4019(AtlRes source, AtlRes target) {
			try {
				if (source == null) {
					return true;
				}
				if (AtlResSuperimpose_4019_SourceExpression == null) {
					Map env = Collections.singletonMap(OPPOSITE_END_VAR, CallchainPackage.eINSTANCE.getAtlRes());
					AtlResSuperimpose_4019_SourceExpression = CallchainOCLFactory.getExpression(
							"self <> oppositeEnd", CallchainPackage.eINSTANCE.getAtlRes(), env); //$NON-NLS-1$
				}
				Object sourceVal = AtlResSuperimpose_4019_SourceExpression.evaluate(source, Collections.singletonMap(
						OPPOSITE_END_VAR, target));
				if (false == sourceVal instanceof Boolean || !((Boolean) sourceVal).booleanValue()) {
					return false;
				} // else fall-through
				if (target == null) {
					return true;
				}
				if (AtlResSuperimpose_4019_TargetExpression == null) {
					Map env = Collections.singletonMap(OPPOSITE_END_VAR, CallchainPackage.eINSTANCE.getAtlRes());
					AtlResSuperimpose_4019_TargetExpression = CallchainOCLFactory.getExpression(
							"true", CallchainPackage.eINSTANCE.getAtlRes(), env); //$NON-NLS-1$
				}
				Object targetVal = AtlResSuperimpose_4019_TargetExpression.evaluate(target, Collections.singletonMap(
						OPPOSITE_END_VAR, source));
				if (false == targetVal instanceof Boolean || !((Boolean) targetVal).booleanValue()) {
					return false;
				} // else fall-through
				return true;
			} catch (Exception e) {
				CallchainDiagramEditorPlugin.getInstance().logError("Link constraint evaluation error", e); //$NON-NLS-1$
				return false;
			}
		}

		/**
		 * @generated
		 */
		public static boolean canExistCallIn_4020(Call source, Model target) {
			try {
				if (source == null) {
					return true;
				}
				if (CallIn_4020_SourceExpression == null) {
					Map env = Collections.singletonMap(OPPOSITE_END_VAR, CallchainPackage.eINSTANCE.getModel());
					CallIn_4020_SourceExpression = CallchainOCLFactory.getExpression(
							"true", CallchainPackage.eINSTANCE.getCall(), env); //$NON-NLS-1$
				}
				Object sourceVal = CallIn_4020_SourceExpression.evaluate(source, Collections.singletonMap(
						OPPOSITE_END_VAR, target));
				if (false == sourceVal instanceof Boolean || !((Boolean) sourceVal).booleanValue()) {
					return false;
				} // else fall-through
				if (target == null) {
					return true;
				}
				if (CallIn_4020_TargetExpression == null) {
					Map env = Collections.singletonMap(OPPOSITE_END_VAR, CallchainPackage.eINSTANCE.getCall());
					CallIn_4020_TargetExpression = CallchainOCLFactory.getExpression(
							"not oppositeEnd.oclIsTypeOf(ModelAlias)", CallchainPackage.eINSTANCE.getModel(), env); //$NON-NLS-1$
				}
				Object targetVal = CallIn_4020_TargetExpression.evaluate(target, Collections.singletonMap(
						OPPOSITE_END_VAR, source));
				if (false == targetVal instanceof Boolean || !((Boolean) targetVal).booleanValue()) {
					return false;
				} // else fall-through
				return true;
			} catch (Exception e) {
				CallchainDiagramEditorPlugin.getInstance().logError("Link constraint evaluation error", e); //$NON-NLS-1$
				return false;
			}
		}

		/**
		 * @generated
		 */
		public static boolean canExistCallOut_4021(Call source, Model target) {

			return true;
		}

		/**
		 * @generated
		 */
		public static boolean canExistCallRes_4022(Call source, AtlRes target) {

			return true;
		}

		/**
		 * @generated
		 */
		public static boolean canExistCallLibarys_4023(Call source, AtlLibary target) {

			return true;
		}

		/**
		 * @generated
		 */
		public static boolean canExistCallArtifact_4024(Call source, Artifact target) {

			return true;
		}

		/**
		 * @generated
		 */
		public static boolean canExistCommentDocumentedElement_4025(Comment source, Documentable target) {

			return true;
		}

		/**
		 * @generated
		 */
		public static boolean canExistGeneratorUses_4026(Generator source, Artifact target) {
			try {
				if (source == null) {
					return true;
				}
				if (GeneratorUses_4026_SourceExpression == null) {
					Map env = Collections.singletonMap(OPPOSITE_END_VAR, CallchainPackage.eINSTANCE.getArtifact());
					GeneratorUses_4026_SourceExpression = CallchainOCLFactory.getExpression(
							"true", CallchainPackage.eINSTANCE.getGenerator(), env); //$NON-NLS-1$
				}
				Object sourceVal = GeneratorUses_4026_SourceExpression.evaluate(source, Collections.singletonMap(
						OPPOSITE_END_VAR, target));
				if (false == sourceVal instanceof Boolean || !((Boolean) sourceVal).booleanValue()) {
					return false;
				} // else fall-through
				if (target == null) {
					return true;
				}
				if (GeneratorUses_4026_TargetExpression == null) {
					Map env = Collections.singletonMap(OPPOSITE_END_VAR, CallchainPackage.eINSTANCE.getGenerator());
					GeneratorUses_4026_TargetExpression = CallchainOCLFactory.getExpression(
							"true", CallchainPackage.eINSTANCE.getArtifact(), env); //$NON-NLS-1$
				}
				Object targetVal = GeneratorUses_4026_TargetExpression.evaluate(target, Collections.singletonMap(
						OPPOSITE_END_VAR, source));
				if (false == targetVal instanceof Boolean || !((Boolean) targetVal).booleanValue()) {
					return false;
				} // else fall-through
				return true;
			} catch (Exception e) {
				CallchainDiagramEditorPlugin.getInstance().logError("Link constraint evaluation error", e); //$NON-NLS-1$
				return false;
			}
		}

		/**
		 * @generated
		 */
		public static boolean canExistGeneratorProduce_4027(Generator source, Artifact target) {
			try {
				if (source == null) {
					return true;
				}
				if (GeneratorProduce_4027_SourceExpression == null) {
					Map env = Collections.singletonMap(OPPOSITE_END_VAR, CallchainPackage.eINSTANCE.getArtifact());
					GeneratorProduce_4027_SourceExpression = CallchainOCLFactory.getExpression(
							"true", CallchainPackage.eINSTANCE.getGenerator(), env); //$NON-NLS-1$
				}
				Object sourceVal = GeneratorProduce_4027_SourceExpression.evaluate(source, Collections.singletonMap(
						OPPOSITE_END_VAR, target));
				if (false == sourceVal instanceof Boolean || !((Boolean) sourceVal).booleanValue()) {
					return false;
				} // else fall-through
				if (target == null) {
					return true;
				}
				if (GeneratorProduce_4027_TargetExpression == null) {
					Map env = Collections.singletonMap(OPPOSITE_END_VAR, CallchainPackage.eINSTANCE.getGenerator());
					GeneratorProduce_4027_TargetExpression = CallchainOCLFactory.getExpression(
							"true", CallchainPackage.eINSTANCE.getArtifact(), env); //$NON-NLS-1$
				}
				Object targetVal = GeneratorProduce_4027_TargetExpression.evaluate(target, Collections.singletonMap(
						OPPOSITE_END_VAR, source));
				if (false == targetVal instanceof Boolean || !((Boolean) targetVal).booleanValue()) {
					return false;
				} // else fall-through
				return true;
			} catch (Exception e) {
				CallchainDiagramEditorPlugin.getInstance().logError("Link constraint evaluation error", e); //$NON-NLS-1$
				return false;
			}
		}

		/**
		 * @generated
		 */
		public static boolean canExistCallableNext_4028(Callable source, Callable target) {
			try {
				if (source == null) {
					return true;
				}
				if (CallableNext_4028_SourceExpression == null) {
					Map env = Collections.singletonMap(OPPOSITE_END_VAR, CallchainPackage.eINSTANCE.getCallable());
					CallableNext_4028_SourceExpression = CallchainOCLFactory.getExpression(
							"self <> oppositeEnd", CallchainPackage.eINSTANCE.getCallable(), env); //$NON-NLS-1$
				}
				Object sourceVal = CallableNext_4028_SourceExpression.evaluate(source, Collections.singletonMap(
						OPPOSITE_END_VAR, target));
				if (false == sourceVal instanceof Boolean || !((Boolean) sourceVal).booleanValue()) {
					return false;
				} // else fall-through
				if (target == null) {
					return true;
				}
				if (CallableNext_4028_TargetExpression == null) {
					Map env = Collections.singletonMap(OPPOSITE_END_VAR, CallchainPackage.eINSTANCE.getCallable());
					CallableNext_4028_TargetExpression = CallchainOCLFactory.getExpression(
							"true", CallchainPackage.eINSTANCE.getCallable(), env); //$NON-NLS-1$
				}
				Object targetVal = CallableNext_4028_TargetExpression.evaluate(target, Collections.singletonMap(
						OPPOSITE_END_VAR, source));
				if (false == targetVal instanceof Boolean || !((Boolean) targetVal).booleanValue()) {
					return false;
				} // else fall-through
				return true;
			} catch (Exception e) {
				CallchainDiagramEditorPlugin.getInstance().logError("Link constraint evaluation error", e); //$NON-NLS-1$
				return false;
			}
		}

	}

}
