/*
 * (c) urs zeidler
 */
package de.urszeidler.eclipse.callchain.executors;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.resources.WorkspaceJob;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.m2m.atl.adt.launching.AtlLauncherTools;
import org.eclipse.m2m.atl.adt.launching.AtlRegularVM;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;

import de.urszeidler.eclipse.callchain.AtlLibary;
import de.urszeidler.eclipse.callchain.AtlRes;
import de.urszeidler.eclipse.callchain.Call;
import de.urszeidler.eclipse.callchain.Callable;
import de.urszeidler.eclipse.callchain.GMF_Generator;
import de.urszeidler.eclipse.callchain.Generator;
import de.urszeidler.eclipse.callchain.Generic_Generator;
import de.urszeidler.eclipse.callchain.Model;
import de.urszeidler.eclipse.callchain.ModelAlias;
import de.urszeidler.eclipse.callchain.TopcasedGenerator;
import de.urszeidler.eclipse.callchain.generatorservice.GeneratorDescriptor;
import de.urszeidler.eclipse.callchain.generatorservice.GeneratorExecutable;
import de.urszeidler.eclipse.callchain.generatorservice.Generatorservice;
import de.urszeidler.eclipse.callchain.provider.CallchainEditPlugin;
import de.urszeidler.eclipse.callchain.ui.emf.actions.CallchainDoGenerationSwitch;
import de.urszeidler.eclipse.callchain.util.CallchainSwitch;

/**
 * this one manages the transformation calls
 * 
 * @author urs
 * 
 */
public class MCcallchain {
	private class ExecutorBuilder extends CallchainSwitch<ExecutorStep> {
		@Override
		public ExecutorStep caseGMF_Generator(GMF_Generator object) {
			return new GMF_GeneratorExe(object);
		}

		@Override
		public ExecutorStep caseTopcasedGenerator(TopcasedGenerator object) {
			return new TopcasedGeneratorExe(object);
		}

		@Override
		public ExecutorStep caseGeneric_Generator(Generic_Generator object) {
			GeneratorDescriptor descriptor = Generatorservice.getDefault()
					.getGenerator(object.getId());
			if (descriptor != null) {
				GeneratorExecutable executor;
				try {
					executor = descriptor.createExecutable();
					return new GenericExecutorStep(executor, object);
				} catch (InstantiationException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (IllegalAccessException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
			return null;
		}
	}

	/**
	 * @author urs executes the atl call
	 */
	private class CallExecutorStep implements ExecutorStep {

		private final Call call;

		public CallExecutorStep(Call object) {
			this.call = object;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see
		 * de.urszeidler.eclipse.callchain.executors.ExecutorStep#generate(java
		 * .lang.Object)
		 */
		@Override
		public void generate(Object monitor) {
			launchCall(call, (IProgressMonitor) monitor);

		}

		@Override
		public String getTaskName() {
			return call.getName();
		}

	}

	public static final String ATT_URI = "uri";
	private final List<Model> tmpModels = new ArrayList<Model>();
	private CallchainDoGenerationSwitch callchainDoGenerationSwitch = new CallchainDoGenerationSwitch();
	private ExecutorBuilder executorBuilder = new ExecutorBuilder();

	public MCcallchain() {
	}

	/**
	 * 
	 * @param callableToStart
	 * @param skipplist
	 * @return
	 */
	private List<ExecutorStep> buildExecutionList(
			final Callable callableToStart, final List<String> skipplist) {
		final List<ExecutorStep> list = new ArrayList<ExecutorStep>();
		Callable lcall = callableToStart;

		while (lcall != null) {

			CallchainSwitch<Object> swt = new CallchainSwitch<Object>() {
				@Override
				public Object caseGenerator(Generator object) {
					if (!skipplist.contains(object.getName())) {
						ExecutorStep step = executorBuilder.doSwitch(object);
						if (step != null)
							list.add(step);

					}
					return null;
				}

				@Override
				public Object caseCall(Call object) {
					if (!skipplist.contains(object.getName()))
						list.add(new CallExecutorStep(object));
					return null;
				}
			};
			swt.doSwitch(lcall);
			lcall = lcall.getNext();
		}
		return list;
	}

	private Shell getShell() {
		Shell shell = null;
		try {
			if (shell == null)
				if (Display.getCurrent() != null)
					shell = Display.getCurrent().getActiveShell();

		} catch (Exception e) {
			// TODO: handle exception
		}
		// try {
		//
		// if (shell == null)
		// if (Display.getDefault() != null)
		// shell = Display.getDefault().getActiveShell();
		//
		// } catch (Exception e) {
		// // TODO: handle exception
		// }
		// try {
		// if (shell == null)
		// shell = PlatformUI.getWorkbench().getActiveWorkbenchWindow()
		// .getShell();
		//
		// } catch (Exception e) {
		// // TODO: handle exception
		// }
		// try {
		//
		// if (shell == null)
		// shell = PlatformUI.getWorkbench().getDisplay().getActiveShell();
		//
		// } catch (Exception e) {
		// // TODO: handle exception
		// }
		// try {
		//
		// if (shell == null)
		// shell = PlatformUI.getWorkbench().getDisplay().getShells()[0];
		//
		// } catch (Exception e) {
		// // TODO: handle exception
		// }

		try {

			// boolean disposeShell = (shell==null);
			// if (disposeShell) {
			// Display display = new Display();
			// shell = new Shell(display);
			// }

			shell = PlatformUI.getWorkbench().getDisplay().getActiveShell();

		} catch (Exception e) {
			// TODO: handle exception
		}

		return shell;
	}

	/**
	 * 
	 * @param callableToStart
	 * @param skipplist
	 */
	public void executeTransformations(final Callable callableToStart,
			final List<String> skipplist) {

		Job job = new WorkspaceJob("Start transformation :") {
			public IStatus runInWorkspace(IProgressMonitor monitor) {

				final List<ExecutorStep> buildExecutionList = buildExecutionList(
						callableToStart, skipplist);
				monitor.beginTask("Start transformation ", buildExecutionList
						.size());
				for (ExecutorStep executorStep : buildExecutionList) {
					if (monitor.isCanceled())
						break;

					monitor.subTask(executorStep.getTaskName());
					try {
						executorStep.generate(monitor);

					} finally {
						monitor.worked(1);
					}
				}

				monitor.done();
				return Status.OK_STATUS;
			}
		};
		job.setUser(true);
		job.schedule();
	}

	@SuppressWarnings("unchecked")
	public void startTransformation(final Callable callableToStart,
			final List<String> skipplist) {
		tmpModels.clear();
		callchainDoGenerationSwitch.setMonitor(null);
		Job job = new WorkspaceJob("execute :") {
			public IStatus runInWorkspace(IProgressMonitor monitor) {
				final IProgressMonitor pmon = monitor;
				//
				try {
					Callable lcall = callableToStart;

					while (lcall != null) {
						if (monitor.isCanceled())
							break;

						CallchainSwitch<Object> swt = new CallchainSwitch<Object>() {
							@Override
							public Object caseGenerator(Generator object) {
								if (!skipplist.contains(object.getName()))
									startGenerator(object, pmon);
								return super.caseGenerator(object);
							}

							@Override
							public Object caseCall(Call object) {
								if (!skipplist.contains(object.getName()))
									launchCall(object, pmon);
								return super.caseCall(object);
							}
						};
						swt.doSwitch(lcall);

						// Job job = jobTheCall(call);
						// try {
						// job.join();
						// } catch (InterruptedException e) {
						// // TODO Auto-generated catch block
						// e.printStackTrace();
						// }
						lcall = lcall.getNext();
					}
					deletetmpFiles(tmpModels);

				} finally {
					pmon.done();
				}
				return Status.OK_STATUS;
			}
		};
		job.setUser(true);

		job.schedule();
	}

	public void startSingleTransformation(final Call call) {
		tmpModels.clear();
		Job job = new Job("execute :" + call.getName()) {
			public IStatus run(IProgressMonitor monitor) {
				Call lcall = call;
				launchCall(lcall, monitor);

				deletetmpFiles(tmpModels);
				return Status.OK_STATUS;
			}
		};
		job.setSystem(true);
		job.schedule();
	}

	/**
	 * delete the tmp models
	 * 
	 * @param tmpModels2
	 */
	protected void deletetmpFiles(List<Model> tmpModels2) {
		IWorkspace wks = ResourcesPlugin.getWorkspace();
		IWorkspaceRoot wksroot = wks.getRoot();

		for (Model model : tmpModels2) {
			IFile file = wksroot.getFile(new Path(model.getUri()));
			if (file.exists())
				try {
					CallchainEditPlugin.log("deleteting tmp Model :"
							+ file.toString(), IStatus.OK);
					file.delete(true, null);
				} catch (CoreException e) {
					CallchainEditPlugin.log(
							"Error while deleteting tmp Model :"
									+ file.toString(), IStatus.WARNING, e);
					e.printStackTrace();
				}
		}

	}

	// /**
	// * to get arraout the IOConsolePartitioner issue
	// *
	// * @param call
	// * @return
	// * @deprecated
	// */
	// private Job jobTheCall(final Call call) {
	// Job job = new Job("execute :" + call.getName()) {
	// public IStatus run(IProgressMonitor monitor) {
	// launchCall(call, monitor);
	// return Status.OK_STATUS;
	// }
	// };
	// job.setSystem(true);
	// job.schedule();
	// return job;
	// }
	//
	/**
	 * start a generator call
	 * 
	 * @param object
	 * @param pmon
	 */
	private void startGenerator(Generator object, IProgressMonitor pmon) {
		callchainDoGenerationSwitch.setMonitor(pmon);
		callchainDoGenerationSwitch.doSwitch(object);

	}

	/**
	 * 
	 * @param call
	 * @param pmon
	 */
	protected void launchCall(Call call, IProgressMonitor pmon) {
		// if (pmon != null)
		// pmon.beginTask(call.getName(), 1);

		String filePath = call.getRes().getUri();
		Map libsFromConfig = collectLibs(call);
		Map input = collectModel2MM(call.getIn());
		Map output = collectModel2MM(call.getOut());
		Map path = collectPath(call);
		Map modelType = collectModelTypes(call);
		Map modelHandler = collectModelHandler(call);
		String mode = "run";
		boolean checkSameModel = !call.isAllowInterModelReferences();// false;
		List superimpose = collectSuperImposed(call);// Collections.EMPTY_LIST;
		boolean continueAfterErrors = call.isContinueAfterErrors();
		Map<String, String> options = new HashMap<String, String>();
		options.put(AtlLauncherTools.OPTION_CONTINUE_AFTER_ERROR, call
				.isContinueAfterErrors() ? "true" : "false");
		options.put(AtlLauncherTools.OPTION_PRINT_EXECUTION_TIME, call
				.isPrintExecutionTime() ? "true" : "false");
		options.put(AtlLauncherTools.OPTION_SHOW_SUMMARY,
				call.isShowSummary() ? "true" : "false");
		options.put(AtlLauncherTools.OPTION_PROFILE, call.isProfile() ? "true"
				: "false");
		// options.put(AtlLauncherTools.OPTION_CONTINUE_AFTER_ERROR,
		// call.isContinueAfterErrors() ? "true" : "false");

		collectTmpModels(call.getIn());
		if (call.getEnviorment().isLogDefault())
			CallchainEditPlugin.log("execute transformation :"
					+ filePath.toString(), IStatus.INFO);

		try {
			URL url = fileNameToURL(filePath);
			AtlRegularVM.runAtlLauncher(filePath, libsFromConfig, input,
					output, path, modelType, modelHandler, mode, superimpose,
					options);

			// AtlLaunchConfigurationDelegate.runAtlLauncher(filePath,
			// libsFromConfig,
			// input, output, path, modelType, modelHandler, mode,
			// checkSameModel, superimpose, continueAfterErrors);
			// org.atl.eclipse.engine.AtlLauncher.getDefault().launch
			// (url, libsFromConfig,
			// input, output, path, modelType, modelHandler,// mode,
			// checkSameModel,
			// superimpose, continueAfterErrors);

			if (call.getArtifact() != null)
				ResourcesPlugin.getWorkspace().getRoot().refreshLocal(2, null);

		} catch (Exception e) {

			CallchainEditPlugin.log("execute transformation :"
					+ filePath.toString(), IStatus.ERROR, e);

		}
		// finally {
		// pmon.done();
		// }
	}

	/**
	 * collect all temp models
	 * 
	 * @param list
	 */
	private void collectTmpModels(EList list) {
		for (Object o : list) {
			if (o instanceof Model && !(o instanceof ModelAlias)) {
				Model model = (Model) o;
				if (model.isIsTemporary())
					tmpModels.add(model);
			}// end of if (o instanceof Model )

		}

	}

	/**
	 * 
	 * @param call
	 * @return
	 */
	@SuppressWarnings( { "unused", "unchecked" })
	private List collectSuperImposed(Call call) {
		ArrayList list = new ArrayList();
		AtlRes atlRes = call.getRes();
		if (atlRes != null) {
			EList superimpose = atlRes.getSuperimpose();
			superimpose.addAll(call.getEnviorment().getDefaultSuperImposed());
			for (Object object : superimpose) {
				if (object instanceof AtlRes) {
					AtlRes satl = (AtlRes) object;
					String atlFile = satl.getUri();
					String currentAsmPath = toASMresource(atlFile);
					list.add(currentAsmPath);
				}// end of if (object instanceof AtlRes)
			}
		}

		return list;
	}

	/**
	 * 
	 * @param atlFile
	 * @return
	 */
	private String toASMresource(String atlFile) {
		String currentAsmPath = atlFile.substring(0, atlFile.length() - 3)
				+ "asm";
		return currentAsmPath;
	}

	/**
	 * collectes the ModelHandler
	 * 
	 * @param call
	 * @return
	 */
	@SuppressWarnings("unchecked")
	private Map collectModelHandler(Call call) {
		Map path = new HashMap();
		List in = call.getIn();
		List out = call.getOut();

		for (int i = 0; i < in.size(); i++) {
			EObject eo = (EObject) in.get(i);
			if (eo instanceof Model) {
				Model m = (Model) eo;
				path.put(m.getName(), "");
				path.put(m.getMm().getName(), m.getMm().getHandler()
						.getLiteral());
				// registerInModel(map, info, eo);
			}// end of if (eo instanceof Model)
		}// end of for i to in
		for (int i = 0; i < out.size(); i++) {
			EObject eo = (EObject) out.get(i);
			if (eo instanceof Model) {
				Model m = (Model) eo;
				path.put(m.getName(), "");
				path.put(m.getMm().getName(), m.getMm().getHandler()
						.getLiteral());
				// registerInModel(map, info, eo);
			}// end of if (eo instanceof Model)
		}// end of for i to out
		return path;
	}

	@SuppressWarnings("unchecked")
	private Map collectModelTypes(Call call) {
		Map path = new HashMap();
		List in = call.getIn();
		List out = call.getOut();

		for (int i = 0; i < in.size(); i++) {
			EObject eo = (EObject) in.get(i);
			if (eo instanceof Model) {
				Model m = (Model) eo;
				path.put(m.getName(), "MODELINPUT");
				path.put(m.getMm().getName(), "METAMODELINPUT");
				// registerInModel(map, info, eo);
			}// end of if (eo instanceof Model)
		}// end of for i to in
		for (int i = 0; i < out.size(); i++) {
			EObject eo = (EObject) out.get(i);
			if (eo instanceof Model) {
				Model m = (Model) eo;
				path.put(m.getName(), "MODELOUTPUT");
				path.put(m.getMm().getName(), "METAMODELOUTPUT");
				// registerInModel(map, info, eo);
			}// end of if (eo instanceof Model)
		}// end of for i to out
		return path;
	}

	@SuppressWarnings("unchecked")
	private Map collectPath(Call call) {
		Map path = new HashMap();

		List in = call.getIn();
		collectPath(path, in);
		List out = call.getOut();
		collectPath(path, out);
		return path;
	}

	/**
	 * @param path
	 * @param in
	 */
	@SuppressWarnings("unchecked")
	private void collectPath(Map path, List in) {
		for (int i = 0; i < in.size(); i++) {
			EObject eo = (EObject) in.get(i);
			if (eo instanceof Model) {
				Model m = (Model) eo;
				path.put(m.getName(), m.getUri());
				path.put(m.getMm().getName(), m.getMm().getUri());
				// registerInModel(map, info, eo);
			}// end of if (eo instanceof Model)
		}// end of for i to in
	}

	/**
	 * @param map
	 * @param in
	 * @param info
	 * @return
	 */
	@SuppressWarnings("unchecked")
	private Map collectModel2MM(EList in) {
		Map models = new HashMap();
		for (int i = 0; i < in.size(); i++) {
			EObject eo = (EObject) in.get(i);
			if (eo instanceof Model) {
				Model m = (Model) eo;
				models.put(m.getName(), m.getMm().getName());
				// registerInModel(map, info, eo);
			}// end of if (eo instanceof Model)
		}// end of for i to in
		return models;
	}

	// /**
	// * @param call
	// */
	// private void startNextCall(Call call) {
	// Map parameters = new HashMap();
	// Map map = new HashMap();
	//
	// EList in = call.getIn();
	// EList out = call.getOut();
	// AtlRes res = call.getRes();
	//
	// // TransformationInfo info = new TransformationInfo("id","callchain");
	// TransformationInfo info = new TransformationInfo(call.getName(), call
	// .getName());
	//
	// parameters.put("Id", call.getName());
	//
	// for (int i = 0; i < in.size(); i++) {
	// EObject eo = (EObject) in.get(i);
	// if (eo instanceof Model) {
	// registerInModel(map, info, eo);
	// }// end of if (eo instanceof Model)
	// }// end of for i to in
	// for (int i = 0; i < out.size(); i++) {
	// EObject eo = (EObject) out.get(i);
	// if (eo instanceof Model) {
	// registerOutModel(map, info, eo);
	// }// end of if (eo instanceof Model)
	// }// end of for i to in
	// //IPath path = new Path("");
	// //parameters.put("ProjectPath", path);
	// parameters.put("Models", map);
	// parameters.put("IsXml", Boolean.FALSE);
	// // Transformation transformation = new
	// // Transformation(info,map,path,false,null);
	// IProgressMonitor monitor = Platform.getJobManager()
	// .createProgressGroup();
	//
	// start(info, map, false, monitor, call);
	// }
	//
	// @SuppressWarnings("unchecked")
	// private void start(TransformationInfo trans, Map models,
	// boolean xmlModels, IProgressMonitor monitor, Call call) {
	// Map outModels = new HashMap();
	//
	// if (trans != null) {
	// Vector fileToDelete = new Vector();
	// try {
	// Map modelHandlers = new HashMap();
	// Map inputMap = new HashMap();
	// Map outputMap = new HashMap();
	// Map pathMap = new HashMap();
	// Map modelTypeMap = new HashMap();
	//
	// for (Iterator ite = trans.getListInParams(); ite.hasNext();) {
	// String key = (String) ite.next();
	// String mmName = trans.getInParamMetamodel(key);
	// inputMap.put(key, mmName);
	// String inputFilename = (String) models.get(key);
	//
	// if (xmlModels) {
	// if (monitor != null) {
	// monitor.subTask("XML Adapter process ...");
	// monitor.worked(5);
	// }
	// // TODO check, double passage!
	// Xmi1toXmi2 xmi1toxmi2 = new Xmi1toXmi2(inputFilename,
	// inputFilename + ".xmi", "ISO-8859-1", mmName);
	// String result = xmi1toxmi2.runImport();
	// if (result.equals("OK")) {
	// // new File(inputFilename).delete();
	// inputFilename = inputFilename.concat(".xmi");
	// fileToDelete.add(inputFilename);
	// }
	// }
	// if (monitor != null) {
	// monitor
	// .subTask("Preparing transformation parameters ...");
	// monitor.worked(20);
	// }
	// pathMap.put(key, "ext:" + inputFilename);
	// // CLC add : to modify if xml file
	// models.put(key, inputFilename);
	//
	// // in case of an already registered metamodel we should
	// // invoke it through its Uri.
	// String uriAlreadyLoaded = getAlreadyloadedMetaModels(trans
	// .getMetamodelId(mmName));
	// if (uriAlreadyLoaded.compareTo(new String("")) != 0) {
	// pathMap.put(mmName, "uri:" + uriAlreadyLoaded);
	// } else {
	// // else we need to load the metamodel. ATL makes it.
	// //
	//pathMap.put(mmName,"ext:"+trans.getMetamodelFile(mmName).getAbsolutePath()
	// );
	// }
	// modelTypeMap.put(key, "MODELINPUT");
	// modelTypeMap.put(mmName, "METAMODELINPUT");
	// if (trans.getMetamodelHandler(mmName).equals("EMF")) {
	// modelHandlers.put(mmName, AtlModelHandler.AMH_EMF);
	// } else {
	// // modelHandlers.put(mmName,AtlModelHandler.AMH_MDR);
	// }
	// }
	//
	// for (Iterator ite = trans.getListOutParams(); ite.hasNext();) {
	// String key = (String) ite.next();
	// String mmName = trans.getOutParamMetamodel(key);
	// outputMap.put(key, mmName);
	// pathMap.put(key, //pathProject.toString() //+ IPath.SEPARATOR
	// //+
	// (String) models.get(key));
	//
	// // in case of an already registered metamodel we should
	// // invoke it through its Uri.
	// String uriAlreadyLoaded = getAlreadyloadedMetaModels(trans
	// .getMetamodelId(mmName));
	// if (uriAlreadyLoaded.compareTo(new String("")) != 0) {
	// pathMap.put(mmName, "uri:" + uriAlreadyLoaded);
	// } else {
	// // else we need to load the metamodel. ATL makes it.
	// //
	//pathMap.put(mmName,"ext:"+trans.getMetamodelFile(mmName).getAbsolutePath()
	// );
	// }
	// modelTypeMap.put(key, "MODELOUTPUT");
	// modelTypeMap.put(mmName, "METAMODELOUTPUT");
	// if (trans.getMetamodelHandler(mmName).equals("EMF")) {
	// modelHandlers.put(mmName, AtlModelHandler.AMH_EMF);
	// } else {
	// // modelHandlers.put(mmName,AtlModelHandler.AMH_MDR);
	// }
	// }
	// if (monitor != null) {
	// monitor.worked(5);
	// }
	// // CLC : ATL Launch new version, to avoid conflict with editors
	// provideInModelsWithMetamodel(trans, models);
	// provideOutModelsWithMetamodel(trans, models);
	//
	// URL mAsmUrl = null;
	// try {
	// URI uri2 = URI.createURI(call.getRes().getUri());
	//
	// URI uri3 = CommonPlugin.asLocalURI(uri2);
	// File inFile = new File(uri3.toFileString());
	//
	// mAsmUrl = inFile.toURI().toURL();
	// } catch (MalformedURLException e1) {
	// CorePlugin.log(
	// "bad URL conversion on transformation rule file",
	// IStatus.ERROR);
	// CorePlugin.log(e1);
	// }
	// Map params = Collections.EMPTY_MAP;
	// Map libs = new HashMap();// Collections.EMPTY_MAP;
	// libs = collectLibs(call);
	//
	// if (monitor != null) {
	// monitor.subTask("Launching transformation ...");
	// }
	//
	// if (LOGGER.isDebugEnabled()) {
	// LOGGER.debug("start(TransformationInfo, Map, IPath, boolean,
	// IProgressMonitor, Call) - transfoModels=" + transfoModels + ", params=" +
	// params + ", libs=" + libs); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	// }
	//
	// AtlLauncher.getDefault().launch(mAsmUrl, libs, transfoModels,
	// params);
	// if (monitor != null) {
	// monitor.worked(15);
	// monitor.subTask("Saving result model(s) ...");
	// }
	// saveOutModels(trans, models);
	// if (monitor != null) {
	// monitor.worked(5);
	// }
	// // retrieve the models produced by the transformation
	// Iterator it = trans.getListOutParams();
	// while (it.hasNext()) {
	// String key = (String) it.next();
	// outModels.put(key, models.get(key));
	// }
	// } catch (NullPointerException ne) {
	// CorePlugin.log(ne);
	// } catch (IllegalArgumentException ie) {
	// CorePlugin.log(ie);
	// }
	// for (int f = 0; f < fileToDelete.size(); f++) {
	// File tempFile = new File((String) fileToDelete.get(f));
	// if (tempFile.exists()) {
	// tempFile.delete();
	// }
	// }
	// trans = null;
	//
	// }
	//
	// if (LOGGER.isInfoEnabled()) {
	// LOGGER.info("start(TransformationInfo, Map, IPath, boolean,
	// IProgressMonitor, Call) - outModels=" + outModels); //$NON-NLS-1$
	// }
	//
	// //resultModels = outModels;
	//
	// }
	//

	private static URL fileNameToURL(String filePath)
			throws MalformedURLException {
		if (filePath.startsWith("ext:")) {
			File f = new File(filePath.substring(4));
			return f.toURL();
		}

		IWorkspace wks = ResourcesPlugin.getWorkspace();
		IWorkspaceRoot wksroot = wks.getRoot();

		IFile currentLib = wksroot.getFile(new Path(filePath));
		return currentLib.getLocation().toFile().toURL();

	}

	/**
	 * added the libs to the map
	 * 
	 * @param call
	 * @param libs
	 */
	@SuppressWarnings("unchecked")
	private Map collectLibs(Call call) {
		EList libarys = call.getLibarys();
		libarys.addAll(call.getEnviorment().getDefaultLib());
		Map libs = new HashMap();

		for (int i = 0; i < libarys.size(); i++) {
			AtlLibary lib = (AtlLibary) libarys.get(i);
			String asmresource = toASMresource(lib.getUri());
			// lib.getUri()
			libs.put(lib.getName(), asmresource);
			if (true)
				continue;
		}// end of for i to in

		return libs;
	}

	//
	// private String getAlreadyloadedMetaModels(String metamodelName) {
	// String toReturn = new String("");
	// IExtensionPoint metamodelsExtent = Platform.getExtensionRegistry()
	// .getExtensionPoint("org.eclipse.emf.ecore.generated_package");
	// IExtension[] listExtensions = metamodelsExtent.getExtensions();
	// boolean notFound = true;
	// int i = 0;
	// while (i < listExtensions.length && notFound) {
	// IConfigurationElement[] elements = listExtensions[i]
	// .getConfigurationElements();
	// for (int j = 0; j < elements.length && notFound; j++) {
	// if (elements[j].getAttribute(ATT_URI).compareTo(metamodelName) == 0) {
	// notFound = !notFound;
	// toReturn = elements[j].getAttribute(ATT_URI);
	// }
	// }
	// i = i + 1;
	// }
	// return toReturn;
	// }
	//
	// /**
	// * @param map
	// * @param info
	// * @param eo
	// */
	// @SuppressWarnings("unchecked")
	// private void registerInModel(Map map, TransformationInfo info, EObject
	// eo) {
	// Model model = (Model) eo;
	// map.put(model.getName(), model.getUri());
	// info.addInParams(model.getName(), model.getMm().getName());
	// // info.setMetamodelFile(model.getMm().getName(), new File(
	// // URI.createPlatformResourceURI(model.getUri()).toFileString()));
	// info.setMetamodelHandler(model.getMm().getName(), "EMF");
	// info.setMetamodelId(model.getMm().getName(), model.getMm().getUri());
	//
	// if (LOGGER.isInfoEnabled()) {
	// LOGGER.info("registerInModel(Map, TransformationInfo, EObject) - info=" +
	// info); //$NON-NLS-1$
	// }
	// }

	public void startSingleTransformation(Callable callableToStart) {
		CallchainSwitch<Object> swt = new CallchainSwitch<Object>() {
			@Override
			public Object caseGenerator(Generator object) {
				startGenerator(object, null);
				return super.caseGenerator(object);
			}

			@Override
			public Object caseCall(Call object) {
				launchCall(object, null);
				return super.caseCall(object);
			}
		};
		swt.doSwitch(callableToStart);
	}

	public void startTransformation(Callable callableToStart) {
		startTransformation(callableToStart, new ArrayList<String>());
	}

	// /**
	// * @param map
	// * @param info
	// * @param eo
	// */
	// @SuppressWarnings("unchecked")
	// private void registerOutModel(Map map, TransformationInfo info, EObject
	// eo) {
	// Model model = (Model) eo;
	// map.put(model.getName(), model.getUri());
	// info.addOutParams(model.getName(), model.getMm().getName());
	// // info.setMetamodelFile(model.getMm().getName(), new File(
	// // URI.createPlatformResourceURI(model.getUri()).toFileString()));
	// info.setMetamodelHandler(model.getMm().getName(), "EMF");
	// info.setMetamodelId(model.getMm().getName(), model.getMm().getUri());
	//
	// if (LOGGER.isDebugEnabled()) {
	// LOGGER.debug("registerOutModel(Map, TransformationInfo, EObject) - info="
	// + info); //$NON-NLS-1$
	// }
	// }
	//
	// /**
	// * Fills the transfoModels table for ATL transformation with the IN models
	// *
	// * @param trans :
	// * TransformationInfo parameters
	// * @param models :
	// * Models involved in the transformation
	// */
	// @SuppressWarnings("unchecked")
	// private void provideInModelsWithMetamodel(TransformationInfo trans,
	// Map models) {
	// ASMModel modelIn = null;
	// ASMModel metaIn = null;
	// transfoModels.put("MOF", amh.getMof());
	// for (Iterator ite1 = trans.getListInParams(); ite1.hasNext();) {
	// String key = (String) ite1.next();
	// String mmName = trans.getInParamMetamodel(key);
	//
	// URL urlMM = null;
	// // get the metamodel associated
	// try {
	// if (trans.getMetamodelFile(mmName) != null) {
	// urlMM = trans.getMetamodelFile(mmName).toURI().toURL();
	// }
	// } catch (MalformedURLException e1) {
	// CorePlugin.log(e1);
	// }
	// // load metamodel
	// InputStream inStream = null;
	// try {
	// if (urlMM != null) {
	// inStream = urlMM.openStream();
	// }
	// } catch (IOException e) {
	// CorePlugin.log(e);
	// }
	//
	// // define the ASMModel
	// // in case of an already registered metamodel we should invoke it
	// // through its Uri.
	// String uriAlreadyLoaded = getAlreadyloadedMetaModels(trans
	// .getMetamodelId(mmName));
	// if (uriAlreadyLoaded.compareTo(new String("")) != 0) {
	// metaIn = amh.loadModel(mmName, amh.getMof(), "uri:"
	// .concat(uriAlreadyLoaded));
	// }
	// // else we need to load the metamodel. ATL makes it.
	// else {
	// // metaIn = amh.loadModel(mmName, amh.getMof(), inStream);
	// }
	//
	// // Put the ASMModel for the transformation
	// transfoModels.put(mmName, metaIn);
	//
	// URL urlModel = null;
	// // get the metamodel associated
	// InputStream inModelStream = null;
	// try {
	// String uri = (String) models.get(key);
	// // URIConverterImpl impl = new URIConverterImpl();
	// URI uri2 = URI.createURI(uri);
	// // URI uri3 = impl.normalize(uri2);
	// // uri3.toString();
	// URI uri3 = CommonPlugin.asLocalURI(uri2);
	// File inFile = new File(uri3.toFileString());
	// // inModelStream = impl.createInputStream(uri2);
	//
	// // File inFile = new File((String) models.get(key));
	// urlModel = inFile.toURI().toURL();
	//
	// if (LOGGER.isDebugEnabled()) {
	// LOGGER
	// .debug("provideInModelsWithMetamodel(TransformationInfo, Map) - modelIn="
	// + modelIn + ", metaIn=" + metaIn + ", urlModel=" + urlModel + ", inFile="
	// + inFile); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
	// }
	// } catch (MalformedURLException e1) {
	// CorePlugin.log(e1);
	//
	// }
	//
	// try {
	// inModelStream = urlModel.openStream();
	// } catch (IOException e) {
	// CorePlugin.log(e);
	// }
	//
	// if (LOGGER.isDebugEnabled()) {
	// LOGGER
	// .debug("provideInModelsWithMetamodel(TransformationInfo, Map) - key=" +
	// key + ", mmName=" + mmName); //$NON-NLS-1$ //$NON-NLS-2$
	// }
	//
	// // define the ASMModel
	//
	// try {
	// modelIn = amh.loadModel(key, metaIn, inModelStream);
	// transfoModels.put(key, modelIn);
	//
	// } catch (Exception e) {
	// LOGGER.fatal("provideInModelsWithMetamodel(TransformationInfo, Map) -
	// modelIn=" + modelIn + ", metaIn=" + metaIn + ", key=" + key + ", mmName="
	// + mmName + ", urlMM=" + urlMM + ", inStream=" + inStream + ", urlModel="
	// + urlModel + ", inModelStream=" + inModelStream, e); //$NON-NLS-1$
	// //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ //$NON-NLS-6$
	// //$NON-NLS-7$ //$NON-NLS-8$
	//
	// }
	// }
	// }
	//
	// /**
	// * Fills the transfoModels table for ATL transformation with the OUT
	// models
	// *
	// * @param trans :
	// * TransformationInfo parameters
	// * @param models :
	// * Models involved in the transformation
	// * @param pathProject :
	// * Path project given as full path
	// */
	// @SuppressWarnings("unchecked")
	// private void provideOutModelsWithMetamodel(TransformationInfo trans,
	// Map models) {
	// ASMModel modelOut = null;
	// ASMModel metaOut = null;
	// transfoModels.put("MOF", amh.getMof());
	// for (Iterator ite1 = trans.getListOutParams(); ite1.hasNext();) {
	// String key = (String) ite1.next();
	// String mmName = trans.getOutParamMetamodel(key);
	//
	// // in case of an already registered metamodel we should invoke it
	// // through its Uri.
	// String uriAlreadyLoaded = getAlreadyloadedMetaModels(trans
	// .getMetamodelId(mmName));
	// if (uriAlreadyLoaded.compareTo(new String("")) != 0) {
	// metaOut = amh.loadModel(mmName, amh.getMof(), "uri:"
	// .concat(uriAlreadyLoaded));
	// }
	// // else we need to load the metamodel. ATL makes it.
	// else {
	// URL urlMM = null;
	// URL urlModel = null;
	// InputStream inStream = null;
	// try {
	// String uri = (String) models.get(key);
	// // URIConverterImpl impl = new URIConverterImpl();
	// URI uri2 = URI.createURI(uri);
	// // URI uri3 = impl.normalize(uri2);
	// // uri3.toString();
	// URI uri3 = CommonPlugin.asLocalURI(uri2);
	// File inFile = new File(uri3.toFileString());
	// // inModelStream = impl.createInputStream(uri2);
	//
	// // File inFile = new File((String) models.get(key));
	// urlModel = inFile.toURI().toURL();
	// } catch (MalformedURLException e1) {
	// CorePlugin.log(e1);
	//
	// }
	//
	// // get the metamodel associated
	// try {
	// urlMM = trans.getMetamodelFile(mmName).toURI().toURL();
	// } catch (MalformedURLException e1) {
	// CorePlugin.log(e1);
	// }
	// // load metamodel
	// // InputStream inStream = null;
	// try {
	// inStream = urlMM.openStream();
	// } catch (IOException e) {
	// CorePlugin.log(e);
	// }
	// metaOut = amh.loadModel(mmName, amh.getMof(), inStream);
	// }
	//
	// // Put the ASMModel for the transformation
	// transfoModels.put(mmName, metaOut);
	//
	// // define the ASMModel for output
	// try {
	// modelOut = amh.newModel((String) models.get(key), metaOut);
	// transfoModels.put(key, modelOut);
	//
	// } catch (Exception e) {
	// if (LOGGER.isDebugEnabled()) {
	// LOGGER.debug("provideOutModelsWithMetamodel(TransformationInfo, Map,
	// IPath) - modelOut=" + modelOut + ", metaOut=" + metaOut + ", key=" + key
	// + ", mmName=" + mmName); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	// //$NON-NLS-4$
	// }
	// }
	// if (LOGGER.isDebugEnabled()) {
	// LOGGER
	// .debug("provideOutModelsWithMetamodel(TransformationInfo, Map, IPath) -
	// key=" + key + ", mmName=" + mmName); //$NON-NLS-1$ //$NON-NLS-2$
	// }
	//
	// }
	// }
	//
	// /**
	// * Allows to save all the models produced by the transformation in the
	// * targetted project
	// *
	// * @param trans :
	// * TransformationInfo parameters
	// * @param models :
	// * Models involved in the transformation
	// * @param pathProject :
	// * Path project given as full path
	// */
	// private void saveOutModels(TransformationInfo trans, Map models) {
	// if (LOGGER.isDebugEnabled()) {
	// LOGGER
	// .debug("saveOutModels(TransformationInfo, Map) - trans=" + trans + ",
	// models=" + models); //$NON-NLS-1$ //$NON-NLS-2$
	// }
	//
	// ASMModel modelOut = null;
	// for (Iterator ite1 = trans.getListOutParams(); ite1.hasNext();) {
	// String key = (String) ite1.next();
	// modelOut = (ASMModel) transfoModels.get(key);
	//
	// if (LOGGER.isDebugEnabled()) {
	// LOGGER
	// .debug("saveOutModels(TransformationInfo, Map) - trans=" + trans + ",
	// key=" + key); //$NON-NLS-1$ //$NON-NLS-2$
	// }
	//
	// amh.saveModel(modelOut, (String) models.get(key));
	// }
	// }
	//
	// /**
	// * Retrieve the .ecore file corresponding to the given metamodel uri
	// */
	// public static File retrieveMetamodel(String mmId) {
	// File mm = null;
	// boolean found = false;
	// IExtensionRegistry registry = Platform.getExtensionRegistry();
	// IExtensionPoint extensionPoint = registry.getExtensionPoint(CorePlugin
	// .getDefault().getBundle().getSymbolicName(), "MetaModels");
	// IExtension[] extensions = extensionPoint.getExtensions();
	// for (int i = 0; i < extensions.length && !(found); i++) {
	// IExtension extension = extensions[i];
	// IConfigurationElement[] elements = extension
	// .getConfigurationElements();
	// for (int j = 0; j < elements.length && !(found); j++) {
	// if (elements[j].getAttribute("uri").equals(mmId)) {
	// found = true;
	// Bundle bundle = Platform.getBundle(extension
	// .getNamespaceIdentifier());
	// URL location = FileLocator.find(bundle, new Path(
	// elements[j].getAttribute("MetaModelDirectory")),
	// null);
	// try {
	// location = FileLocator.resolve(location);
	// } catch (NullPointerException npe) {
	// // anything else except return null !!!! (no log)
	// return null;
	// } catch (IOException e) {
	// // anything else except return null !!!! (no log)
	// return null;
	// }
	// mm = new File(location.getFile());
	// }
	// }
	// }
	//
	// if (LOGGER.isDebugEnabled()) {
	// LOGGER.debug("retrieveMetamodel(String) - mm=" + mm); //$NON-NLS-1$
	// }
	//
	// return mm;
	// }

}
