/*
 * (c) urs zeidler
 */
package de.urszeidler.eclipse.callchain.lauching.core;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.debug.core.ILaunch;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.model.ILaunchConfigurationDelegate;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.util.Diagnostician;
import org.eclipse.ui.console.IConsole;
import org.eclipse.ui.console.IConsoleManager;
import org.eclipse.ui.console.TextConsole;
import org.eclipse.ui.console.actions.ClearOutputAction;

import de.urszeidler.eclipse.callchain.Call;
import de.urszeidler.eclipse.callchain.Callable;
import de.urszeidler.eclipse.callchain.Calls;
import de.urszeidler.eclipse.callchain.Generator;
import de.urszeidler.eclipse.callchain.executors.MCcallchain;

/**
 * <p>
 * CallchainLaunchConfigurationDelegate
 * </p>
 * Created : 03.06.2008
 * 
 * @author urs
 * @version $Revision$, $Id$
 */
public class CallchainLaunchConfigurationDelegate implements
        ILaunchConfigurationDelegate {

    private static final Object ATL_CONSOLE = "org.eclipse.m2m.atl.adt.editor.console";

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.debug.core.model.ILaunchConfigurationDelegate#launch(org.eclipse.debug.core.ILaunchConfiguration,
     *      java.lang.String, org.eclipse.debug.core.ILaunch,
     *      org.eclipse.core.runtime.IProgressMonitor)
     */
    public void launch(ILaunchConfiguration configuration, String mode,
            ILaunch launch, IProgressMonitor monitor) throws CoreException {

    	
    	
        String callchainUri = configuration.getAttribute(
                CallchainConfiguration.CALLCHAINURI, "");
        String callableName = configuration.getAttribute(
                CallchainConfiguration.CALLABLENAME, "");
        boolean chaincalls = configuration.getAttribute(CallchainConfiguration.CHAINCALLS, false);
        boolean clearConsole =  configuration.getAttribute(CallchainConfiguration.CLEARCONSOLE, true);
        Calls calls = loadCallchain(callchainUri);
        Callable callableToStart = findCallable(callableName, calls);
        List<String> list = configuration.getAttribute(CallchainConfiguration.SKIPPLIST, new ArrayList<String>());
        
        if(clearConsole)
        	clearConsole();

        if (callableToStart == null)
            throw new CoreException(new Status(IStatus.ERROR, "",
                    "no callable To Start found :" + callableName));

        MCcallchain ccallchain = new MCcallchain();
        if (!chaincalls)
            ccallchain.startSingleTransformation(callableToStart);
        else
            //ccallchain.startTransformation(callableToStart,list);
        	ccallchain.executeTransformations(callableToStart,list);

    }

	/**
	 * clears the atl console
	 */
	private void clearConsole() {
		IConsoleManager consoleManager = org.eclipse.ui.console.ConsolePlugin.getDefault().getConsoleManager();
    	IConsole[] consoles = consoleManager.getConsoles();
    	for (int i = 0; i < consoles.length; i++) {
			IConsole console = consoles[i];
			if(console.getName().equals(ATL_CONSOLE)){
				if (console instanceof TextConsole) {
					TextConsole tc = (TextConsole) console;
					tc.clearConsole();
				}
			}
				
		}
	}

    /**
     * 
     * @param callableName
     * @param calls
     * @return
     */
    private Callable findCallable(String callableName, Calls calls) {
        if(callableName==null | "".equals(callableName))
            return null;
        
        callableName = callableName.trim();
        EList<Call> list = calls.getCall();
 //       Callable callableToStart = null;
        for (Call call : list) {
            if (call.getName() != null)
                if (callableName.endsWith(call.getName())) {
                    return call;
                    
                }
        }
            EList<Generator> generators = calls.getGenerators();
            for (Generator generator : generators) {
                if (generator.getName() != null)
                    if (callableName.endsWith(generator.getName())) {
                        return generator;
                    }
            }
         return null;
    }

    public static Calls loadCallchain(String callchainUri) {
        File file = new File(callchainUri);
        URI uri = file.isFile() ? URI.createFileURI(file.getAbsolutePath())
                : URI.createURI(callchainUri);

        try {
            ResourceSetImpl resourceSet = new ResourceSetImpl();
            // Demand load resource for this file.
            //
            Resource resource = resourceSet.getResource(uri, true);
            return (Calls) resource.getContents().get(0);
            // Validate the contents of the loaded resource.
            //

        } catch (RuntimeException exception) {
        	//TODO : handle this properly
            System.err.println("Problem loading " + uri);
            exception.printStackTrace();
           
        }

        return null;

    }

}
