/**
 * 
 */
package de.urszeidler.eclipse.callchain.topcasedgenerators;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Status;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.ecore.util.Diagnostician;
import org.topcased.generator.util.ConfiguratorObjectManager;
import org.topcased.properties.configurator.TabbedView;
import org.topcased.properties.generator.PropertiesViewGenerator;

import de.urszeidler.eclipse.callchain.Generic_Generator;
import de.urszeidler.eclipse.callchain.Model;
import de.urszeidler.eclipse.callchain.generatorservice.GeneratorExecutable;
import de.urszeidler.eclipse.callchain.generatorservice.executables.AbstractExecuteable;

/**
 * @author urs
 * 
 */
public class PropertyGenerator extends AbstractExecuteable implements GeneratorExecutable {

	private static final String TC_PROPERTYCONFIGURATOR = "http://www.topcased.org/PropertiesViewConfiguration/1.0";

	/**
	 * 
	 */
	public PropertyGenerator() {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.urszeidler.eclipse.callchain.generatorservice.GeneratorExecutable#
	 * configure(de.urszeidler.eclipse.callchain.Generic_Generator)
	 */
	@Override
	public void configure(Object generator) {
		super.configure(generator);

	}

	@Override
	protected String validateGG(Generic_Generator generator) {
		Model model = findModelByMetaModel(generator.getUses(), TC_PROPERTYCONFIGURATOR);
		if (model == null)
			return "No used model conforming to "+TC_PROPERTYCONFIGURATOR+" metamodel" ;

		return "";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.urszeidler.eclipse.callchain.generatorservice.GeneratorExecutable#
	 * execute(org.eclipse.core.runtime.IProgressMonitor)
	 */
	@Override
	public void execute(IProgressMonitor monitor) throws CoreException {
		Model model = findModelByMetaModel(generator.getUses(), TC_PROPERTYCONFIGURATOR);
		if (model == null)
			return;

		String uri = model.getUri();
		IWorkspace wks = ResourcesPlugin.getWorkspace();
		IWorkspaceRoot wksroot = wks.getRoot();

		IFile currentFile = wksroot.getFile(new Path(uri));

		try {
			ConfiguratorObjectManager manager = new ConfiguratorObjectManager();
			manager.load(currentFile.getFullPath());
			// retrieve the root model object and check if it has the expected
			// type
			if (manager.getRootModelObject() instanceof TabbedView) {
				TabbedView configuration = (TabbedView) manager.getRootModelObject();
				monitor.subTask("Properties Configuration validation");
				Diagnostician diagnostician = new Diagnostician();
				Diagnostic diagnostic = diagnostician.validate(configuration);
				if (diagnostic.getSeverity() <= Diagnostic.INFO) {
					PropertiesViewGenerator generator = new PropertiesViewGenerator(configuration);
					generator.generate(monitor);
				} else {
					throw new CoreException(new Status(IStatus.ERROR,
							"de.urszeidler.eclipse.callchain.topcasedgenerators",
							"Validation problem : a problem occured during the properties configuration validation."));
				}
			} else {
				throw new CoreException(new Status(IStatus.ERROR, "de.urszeidler.eclipse.callchain.topcasedgenerators",
						"The root model object has not the right type. Generation process has been aborted."));
			}

		} catch (Exception e) {
			throw new CoreException(new Status(IStatus.ERROR, "de.urszeidler.eclipse.callchain.topcasedgenerators", e
					.getMessage(), e));
		}

	}

}
