/**
 * 
 */
package de.urszeidler.eclipse.callchain.topcasedgenerators;

import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.topcased.java.reverse.JavaAnnotaions2UMLConverter;

import de.urszeidler.eclipse.callchain.Artifact;
import de.urszeidler.eclipse.callchain.Generic_Generator;
import de.urszeidler.eclipse.callchain.Model;
import de.urszeidler.eclipse.callchain.generatorservice.GeneratorExecutable;

/**
 * @author urs
 * 
 */
public class UMLProfileImporter extends AbstractTopcasedUMLimporter implements GeneratorExecutable {

	/**
	 * 
	 */
	public UMLProfileImporter() {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.urszeidler.eclipse.callchain.generatorservice.GeneratorExecutable#
	 * configure(de.urszeidler.eclipse.callchain.Generic_Generator)
	 */
	@Override
	public void configure(Object generator) {
		super.configure(generator);
	}
	
	@Override
	protected String validateGG(Generic_Generator generator) {
		Model model =  findModelByMetaModel(generator.getUses(), UML2_MODEL_2_0_0, UML2_MODEL_2_1_0);
		if (model == null)
			return "No used model conforms to "+UML2_MODEL_2_0_0+"or "+UML2_MODEL_2_1_0+" Metamodel.";
		
		model =  findModelByMetaModel(generator.getProduce(), UML2_MODEL_2_0_0, UML2_MODEL_2_1_0);
		if (model == null)
			return "No produced model conforms to "+UML2_MODEL_2_0_0+"or "+UML2_MODEL_2_1_0+" Metamodel.";
		
		Artifact javacode = getFirstArtifact(generator.getUses());
		if (javacode == null)
			return "No artifact defining the java code.";
		
		IWorkspace wks = ResourcesPlugin.getWorkspace();
		IWorkspaceRoot wksroot = wks.getRoot();
		
		String javaname = javacode.getUri();
		IProject project = wksroot.getProject(javaname);
		if (project == null)
			return "Artifact uri not defining a project.";
		
		return "";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.urszeidler.eclipse.callchain.generatorservice.GeneratorExecutable#
	 * execute(org.eclipse.core.runtime.IProgressMonitor)
	 */
	@Override
	public void execute(IProgressMonitor monitor) throws CoreException {
		Artifact javacode = getFirstArtifact(generator.getUses());
		Artifact uml = findModelByMetaModel(generator.getProduce(), UML2_MODEL_2_0_0, UML2_MODEL_2_1_0);
		List<Model> modelList = findAllModels(generator.getUses(), UML2_MODEL_2_0_0, UML2_MODEL_2_1_0);

		String umluri = uml.getUri();
		IWorkspace wks = ResourcesPlugin.getWorkspace();
		IWorkspaceRoot wksroot = wks.getRoot();

		monitor.beginTask("Reverse Engineering", 5);

		String modelName = uml.getName();
		String javaname = javacode.getUri();
		IProject project = wksroot.getProject(javaname);
		IJavaProject jproject = JavaCore.create(project);

		IFile modelFile = wksroot.getFile(new Path(umluri));

		// Reverse Engineering
		JavaAnnotaions2UMLConverter converter = new JavaAnnotaions2UMLConverter();
		String[] importList = convertModelListToUriArray(modelList);
		converter.setImportList(importList);
		org.eclipse.uml2.uml.Package model;
		try {
			try {
				Resource emfresource = createNewResource(modelFile);
				converter.setModelName(modelName);
				model = converter.convert(jproject, emfresource);
				Map options = new HashMap();
				emfresource.save(options);
				// model.setName(modelName);
				// Save reversed UML model
				// saveModelFile(model, modelFile);
			} catch (IOException e) {
				e.printStackTrace();
				// TODO : handle exception
			}
		} catch (JavaModelException e) {
			// TODO : handle exception
		}

	}

}
