/*******************************************************************************
 * Copyright (c) 2005 Urs Zeidler
 * This program is free software, distributed under the terms of 
 * the GNU General Public License Version 2. See the LICENSE file 
 * at the top of the source tree.
 * 
 *
 * Contributors:
 *     Urs Zeidler
 *******************************************************************************/
/*
 * (c) urs zeidler
 */
package de.urszeidler.eclipse.callchain.ui.emf.actions;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gmf.runtime.notation.View;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.widgets.Event;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;

/** 
 * <p> 
 * SelectionAction a custom emf selection class
 * </p> 
 * Created : 19.11.2005 
 * @author urs 
 * @version $Revision: 1.1 $, $Id: SelectionAction.java,v 1.1 2008/04/24 11:40:54 urs Exp $ 
 */
public abstract class SelectionAction extends Action implements ISelectionChangedListener,IObjectActionDelegate{
    protected TreeViewer parentViewer;
    protected EObject selection;
    protected IWorkbenchPart wPart;
    /**
     * 
     */
    public SelectionAction(IWorkbenchPart part) {
        super();
        this.wPart = part;
        initAction();
    }

    /**
     * @param text
     */
    public SelectionAction(String text) {
        super(text);
        initAction();
    }

    /**
     * @param text
     * @param image
     */
    public SelectionAction(String text, ImageDescriptor image) {
        super(text, image);
        initAction();
    }

    /**
     * @param text
     * @param style
     */
    public SelectionAction(String text, int style) {
        super(text, style);
        initAction();
    }

    public EObject getSelection() {
        return this.selection;
    }

    public void setSelection(EObject selection) {
        this.selection = selection;
    }
    
//public boolean isEnabled() {
//    return  selection!=null;
//}

    protected abstract void initAction();

    protected void refreshViewer(){
    	if(parentViewer!=null)
    		parentViewer.refresh();
    }

    public void setParentViewer(TreeViewer parentViewer) {
        this.parentViewer = parentViewer;
    }

    public void selectionChanged(SelectionChangedEvent event) {
       ISelection sel = event.getSelection();
       setInterSelection(sel);
        
    }

	public void setInterSelection(ISelection sel) {
		if (sel instanceof StructuredSelection) {
		    StructuredSelection ss = (StructuredSelection) sel;
		    if (ss.getFirstElement() instanceof EObject) {
		        EObject eo = (EObject) ss.getFirstElement();
		        selection = eo;
		    }else
		    if (ss.getFirstElement() instanceof GraphicalEditPart){
		    	Object model = ((GraphicalEditPart)ss.getFirstElement()).getModel();
		    	if (model instanceof View) {
					View view = (View) model;
					selection = view.getElement();
				}
		    }else		    	
		        selection = null;
		 }//end of if (sel instanceof StructuredSelection)
	}

    /**
     * @return the wPart
     */
    public IWorkbenchPart getWPart() {
        return this.wPart;
    }

    /**
     * @param part the wPart to set
     */
    public void setWPart(IWorkbenchPart part) {
        this.wPart = part;
    }
 //   @Override
	public void setActivePart(IAction arg0, IWorkbenchPart arg1) {
		setWPart(arg1);
		
	}
//	@Override
	public void run(IAction arg0) {
		run();
		
	}
	
	@Override
	public void runWithEvent(Event event) {
    	if(selection==null){
    		if(wPart!=null)
    		setInterSelection(wPart.getSite().getSelectionProvider().getSelection());
    	}

		super.runWithEvent(event);
	}
//	@Override
	public void selectionChanged(IAction arg0, ISelection arg1) {
		setInterSelection(arg1);
		
	}

}
