/*******************************************************************************
 * urs zeidler
 ******************************************************************************/
package de.urszeidler.eclipse.callchain.ui.wizards;

import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.ENamedElement;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.gmf.internal.bridge.resolver.StructureResolver;
import org.eclipse.gmf.internal.bridge.wizards.Messages;
import org.eclipse.gmf.internal.bridge.wizards.pages.simple.DefinitionPage;
import org.eclipse.gmf.internal.bridge.wizards.pages.simple.DomainModelSelectionPage;
import org.eclipse.gmf.internal.bridge.wizards.pages.simple.DomainModelSourceImpl;
import org.eclipse.gmf.internal.bridge.wizards.pages.simple.GraphDefBuilder;
import org.eclipse.gmf.internal.bridge.wizards.pages.simple.ResolvedItem;
import org.eclipse.gmf.internal.bridge.wizards.pages.simple.StructureBuilder;
import org.eclipse.gmf.internal.bridge.wizards.pages.simple.ResolvedItem.Resolution;
import org.eclipse.gmf.internal.common.ui.ResourceLocationProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.ide.IDE;

import de.urszeidler.eclipse.callchain.provider.CallchainEditPlugin;

/**
 * Generated wizard that offers the model creation facilities.
 * 
 * @generated
 */
@SuppressWarnings("restriction")
public class NewCallchainDiagrams_GMFdiagram extends BasicTemplateWizard {
	private static final String BASE_BUNDEL_ENTRY = "/templates/GMF_from_ECORE";

	private IStructuredSelection selection;
	private CallchainSimpleTemplatePage diagPage;
	private IFile createdFile;

	private DomainModelSelectionPage domainModelSelectionPage;

	private DefinitionPage graphicalDefinitionPage;

	/**
	 * @see org.eclipse.ui.IWorkbenchWizard#init(org.eclipse.ui.IWorkbench,
	 *      org.eclipse.jface.viewers.IStructuredSelection)
	 */
	public void init(IWorkbench workbench, IStructuredSelection sel) {
		createdFile = null;
		selection = sel;

		// TODO put the Wizard image
		// setDefaultPageImageDescriptor(WizardsPlugin
		// .getImageDescriptor("NEW_PAGE_WZD"));
		setDialogSettings(WizardsPlugin.getDefault().getDialogSettings());
		setWindowTitle("Create a process to produce a gmf diagram editor.");

	}

	@Override
	protected void initVariables(String name, String cname, String pname) {
		super.initVariables(name, cname, pname);

		_variables.put("canvas", domainModelSelectionPage.getDiagramElement().getName());
		_variables.put("collectedLinkNodes", collectLinkNodes());
		_variables.put("excludedEClasses", collectExcluded(EClass.class));
		_variables.put("excludedEReference", collectExcluded(EReference.class));
		_variables.put("excludedEAttributes", collectExcluded(EAttribute.class));
		_variables.put("linkTarget", collectLinkTargets());

	}

	private String collectLinkTargets() {
		ResolvedItem model = graphicalDefinitionPage.getModel();
		ArrayList<ResolvedItem> excludedElements = new ArrayList<ResolvedItem>();
		collectLinkTargets(model, excludedElements);
		StringBuilder builder = new StringBuilder();
		
		Iterator<ResolvedItem> iterator = excludedElements.iterator();
		for (ResolvedItem resolvedItem : excludedElements) {
			iterator.next();
			String name = null;
				EStructuralFeature esf = (EStructuralFeature) resolvedItem.getDomainRef();
				EClass clazz1 = esf.getEContainingClass();
				name = clazz1.getName() + "." + esf.getName();
			
			
			builder.append("('").append(name).append("','")
			.append(name).append("')");
			if (iterator.hasNext())
				builder.append(",").append("\r\n");

		}

		
		return builder.toString();
	}

	private void collectLinkTargets(ResolvedItem model, ArrayList<ResolvedItem> excludedElements) {
		if (model.getResolution() == Resolution.LINK)
			if (EReference.class.isAssignableFrom(model.getDomainRef().getClass()))
				excludedElements.add(model);

		
		List<ResolvedItem> children = model.getChildren();
		for (ResolvedItem resolvedItem : children) {
			collectLinkTargets(resolvedItem, excludedElements);
		}

	}

	private String collectExcluded(Class<?> clazz) {
		ResolvedItem model = graphicalDefinitionPage.getModel();
		ArrayList<ResolvedItem> excludedElements = new ArrayList<ResolvedItem>();
		collectExcluded(model, excludedElements, clazz);
		StringBuilder builder = new StringBuilder();
		Iterator<ResolvedItem> iterator = excludedElements.iterator();
		for (ResolvedItem resolvedItem : excludedElements) {
			iterator.next();
			String name = null;
			if (resolvedItem.getDomainRef() instanceof EStructuralFeature) {
				EStructuralFeature esf = (EStructuralFeature) resolvedItem.getDomainRef();
				EClass clazz1 = esf.getEContainingClass();
				name = clazz1.getName() + "." + esf.getName();
			} else
				name = ((ENamedElement) resolvedItem.getDomainRef()).getName();
			
			builder.append("'").append(name).append("'");
			if (iterator.hasNext())
				builder.append(",").append("\r\n");

		}

		return builder.toString();
	}

	private void collectExcluded(ResolvedItem model, ArrayList<ResolvedItem> excludedElements, Class<?> clazz) {
		if (model.getResolution() == null)
			if (clazz.isAssignableFrom(model.getDomainRef().getClass())){
			if(clazz==EReference.class){
				if (model.getDomainRef() instanceof EReference) {
					EReference eref = (EReference) model.getDomainRef();
					if(!eref.isContainment())
						excludedElements.add(model);
				}
			}else
				
				excludedElements.add(model);
			}
		List<ResolvedItem> children = model.getChildren();
		for (ResolvedItem resolvedItem : children) {
			collectExcluded(resolvedItem, excludedElements, clazz);
		}

	}

	private String collectLinkNodes() {
		ResolvedItem model = graphicalDefinitionPage.getModel();
		ArrayList<ResolvedItem> linkElements = new ArrayList<ResolvedItem>();
		collectLinkNode(model, linkElements);
		StringBuilder builder = new StringBuilder();
		Iterator<ResolvedItem> iterator = linkElements.iterator();
		for (ResolvedItem resolvedItem : linkElements) {
			iterator.next();
			String name = ((EClass) resolvedItem.getDomainRef()).getName();
			builder.append("('").append(name).append("')");
			if (iterator.hasNext())
				builder.append(",");

		}

		return builder.toString();
	}

	private void collectLinkNode(ResolvedItem model, ArrayList<ResolvedItem> linkElements) {
		if (model.getResolution() == Resolution.LINK)
			if (model.getDomainRef() instanceof EClass)
				linkElements.add(model);

		List<ResolvedItem> children = model.getChildren();
		for (ResolvedItem resolvedItem : children) {
			collectLinkNode(resolvedItem, linkElements);
		}

	}

	/**
	 * @see org.eclipse.jface.wizard.IWizard#performFinish()
	 */
	public boolean performFinish() {
		boolean result = true;
		if (diagPage.isPageComplete()) {
			try {
				result = result & createFilesFromTemplate();

			} catch (CoreException e) {
				CallchainEditPlugin.log(e);
			} catch (MalformedURLException e) {
				CallchainEditPlugin.log(e);
			}
			// result = result & createDiagramFile();
			if (createdFile != null && result) {
				// Open the newly created model
				try {
					IDE.openEditor(
							WizardsPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getActivePage(),
							createdFile, true);
				} catch (PartInitException pie) {
					CallchainEditPlugin.log(pie);
				}
			}
			if (diagPage.isAddAltNature())
				addATLnature(null, diagPage.getNatureType());
		}
		return result;
	}

	/**
	 * @see org.eclipse.jface.wizard.IWizard#addPages()
	 */
	public void addPages() {
		diagPage = new CallchainSimpleTemplatePage("New gmf diagram editor", selection, "ecore");
		diagPage.setTitle("Callchain Diagrams");
		diagPage.setDescription("Select your ecore input model.");
		diagPage.setFileExtension("callchain");
		diagPage.setSourceFileExtension("ecore");
		addPage(diagPage);

		ResourceLocationProvider rloc = new ResourceLocationProvider(selection);
		ResourceSet resourceSet = new ResourceSetImpl();

		domainModelSelectionPage = new DomainModelSelectionPage("DomainModelSelectionPage", rloc, resourceSet); //$NON-NLS-1$
		domainModelSelectionPage.setTitle(Messages.SimpleModelWizardDomainModelSelectionPageTitle);
		domainModelSelectionPage.setDescription(Messages.SimpleModelWizardDomainModelSelectionPageDesc);
		addPage(domainModelSelectionPage);

		graphicalDefinitionPage = new DefinitionPage("GraphicalDefinitionPage", new StructureBuilder(
				new StructureResolver(), true), domainModelSelectionPage);
		graphicalDefinitionPage.setTitle(Messages.SimpleModelWizardGraphDefinitionPageTitle);
		graphicalDefinitionPage.setDescription(Messages.SimpleModelWizardGraphDefinitionPageDesc);
		addPage(graphicalDefinitionPage);

	}

	/**
	 * @see org.eclipse.jface.wizard.IWizard#canFinish()
	 */
	public boolean canFinish() {
		return diagPage.isPageComplete() && domainModelSelectionPage.isPageComplete();
	}

	protected IContainer getTargetContainer() {
		return diagPage.getSelectedIContainer();
	}

	protected String getModelName() {
		return diagPage.getModelName();
	}

	protected String getBaseTemplatePath() {
		return BASE_BUNDEL_ENTRY;
	}

	@Override
	protected String getProjectName() {
		return diagPage.getProjectName();
	}

}
